#define SDL_MAIN_USE_CALLBACKS
#include <SDL3/SDL.h>
#include <SDL3/SDL_main.h>

#include "dat.h"

void
SDL_AppQuit(void *as, SDL_AppResult res)
{
	State *st;

	st = as;
	if (st) {
		SDL_Log("destroy renderer...");
		SDL_DestroyRenderer(st->ren);
		SDL_Log("destroy window...");
		SDL_DestroyWindow(st->win);
		SDL_Log("free state...");
		SDL_free(st);
	}

	SDL_Log("bye!");
	return;
}

SDL_AppResult
SDL_AppEvent(void *as, SDL_Event *ev)
{
	switch (ev->type) {
	case SDL_EVENT_QUIT:
		SDL_Log("quit requested");
		return SDL_APP_SUCCESS;
		break; /* unreachable; here for visual symmetry */
	}

	return SDL_APP_CONTINUE;
}

SDL_AppResult
SDL_AppIterate(void *as)
{
	return SDL_APP_CONTINUE;
}

SDL_AppResult
SDL_AppInit(void **as, int argc, char **argv)
{
	State *st;

	SDL_Log("init sdl...");
	if (!SDL_Init(SDL_INIT_VIDEO)) {
		SDL_LogCritical(SDL_LOG_CATEGORY_APPLICATION,
				"sdl init fail: %s", SDL_GetError());
		return SDL_APP_FAILURE;
	}

	SDL_Log("alloc state...");
	st = SDL_calloc(1, sizeof(State));
	if (!st) {
		SDL_LogCritical(SDL_LOG_CATEGORY_APPLICATION,
				"mem alloc fail: %s", SDL_GetError());
		return SDL_APP_FAILURE;
	}
	*as = st;

	SDL_Log("create win+render...");
	if (!SDL_CreateWindowAndRenderer("slutpet", 320, 240,
				SDL_WINDOW_TRANSPARENT | SDL_WINDOW_BORDERLESS,
				&st->win, &st->ren)) {
		SDL_LogCritical(SDL_LOG_CATEGORY_APPLICATION,
				"win creation fail: %s", SDL_GetError());
		return SDL_APP_FAILURE;
	}

	SDL_Log("it work");
	return SDL_APP_CONTINUE;
}
